﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Text;
using System.Windows.Forms;
using Tomham;
using Tomham.Collections.Specialized;
using Tomham.Windows.Forms;

namespace CommandTagLauncher
{
    public partial class CommandEditForm : Form
    {
        private readonly string sampleTagsFileName =
            Path.Combine( Application.StartupPath, @"SampleTags.txt" );

        private readonly string tagsFileName =
            Path.Combine( Application.StartupPath, @"Tags.txt" );

        private List<Command> childCommandsReferencesResolveSource;
        private FileCommand fileCommand;
        private GroupCommand groupCommand;

        public CommandEditForm()
        {
            InitializeComponent();
        }

        public void SetChildCommandsReferencesResolveSource( List<Command> childCommandsReferencesSource )
        {
            childCommandsReferencesResolveSource = childCommandsReferencesSource;
        }

        public void SetCommand( Command command )
        {
            if ( command is FileCommand )
            {
                SetFileCommandMode( (FileCommand) command );
            }
            else
            {
                SetGroupCommandMode( (GroupCommand) command );
            }
        }

        private void SetGroupCommandMode( GroupCommand command )
        {
            groupCommand = command;

            UpdateFromCommand( command );

            UpdateFromGroupCommand( command );

            ShowGroupCommandSpecializedPanel();
        }

        private void UpdateFromGroupCommand( GroupCommand command )
        {
            foreach ( Command member in command.Items )
            {
                labeledTextBoxGroupMembers.TextBox.AppendText( member.Name + "\r\n" );
            }
        }

        private void SetFileCommandMode( FileCommand command )
        {
            fileCommand = command;

            UpdateFromCommand( command );

            UpdateFromFileCommand( command );

            ShowFileCommandSpecializedPanel();
        }

        private void UpdateFromFileCommand( FileCommand command )
        {
            labeledTextBoxFileName.TextBox.Text = command.FileName;
            labeledTextBoxVerb.TextBox.Text = command.Verb;
            labeledTextBoxWorkingDirectory.TextBox.Text = command.WorkingDirectory;
            labeledComboBoxWindowStyle.ComboBox.SelectedIndex = (int) command.WindowStyle;
            labeledComboBoxProcessPriority.ComboBox.SelectedIndex = (int) command.ProcessPriority;

            checkBoxMultipleExecution.Checked = command.MultipleExecution;
        }

        private void UpdateFromCommand( Command command )
        {
            labeledTextBoxName.TextBox.Text = command.Name;
            labeledTextBoxDefaultArguments.TextBox.Text = command.DefaultArguments;
            labeledTextBoxDescription.TextBox.Text = command.Description;

            foreach ( string tag in command.Tags )
            {
                labeledTextBoxTags.TextBox.AppendText( tag + "\r\n" );
            }
        }

        private void SetResultAsCommand( Command command )
        {
            command.Name = labeledTextBoxName.TextBox.Text;
            command.DefaultArguments = labeledTextBoxDefaultArguments.TextBox.Text;
            command.Description = labeledTextBoxDescription.TextBox.Text;

            command.Tags = GetNormalizedItems( labeledTextBoxTags.TextBox.Lines );
        }

        private List<string> GetNormalizedItems( string[] source )
        {
            List<string> result = new List<string>();

            foreach ( string text in source )
            {
                string trimmedText = text.Trim();

                if ( trimmedText != string.Empty )
                {
                    result.Add( trimmedText );
                }
            }

            return result;
        }

        private void SetResultAsFileCommand( FileCommand command )
        {
            command.FileName = labeledTextBoxFileName.TextBox.Text;
            command.Verb = labeledTextBoxVerb.TextBox.Text;
            command.WorkingDirectory = labeledTextBoxWorkingDirectory.TextBox.Text;
            command.WindowStyle = (ProcessWindowStyle) labeledComboBoxWindowStyle.ComboBox.SelectedIndex;
            command.ProcessPriority = (ProcessPriority) labeledComboBoxProcessPriority.ComboBox.SelectedIndex;

            command.MultipleExecution = checkBoxMultipleExecution.Checked;
        }

        private void SetResultAsGroupCommand( GroupCommand command )
        {
            command.Items.Clear();

            List<string> childNames = GetNormalizedItems( labeledTextBoxGroupMembers.TextBox.Lines );

            ResolveChildCommandsReferences( command, childNames );
        }

        private void ResolveChildCommandsReferences( GroupCommand command, List<string> childNames )
        {
            foreach ( string childName in childNames )
            {
                Command child = Commands.Find( childName, childCommandsReferencesResolveSource );

                if ( child == null )
                {
                    string message = childName + " というコマンドが見つかりませんでした。";

                    MessageBox.Show( message );
                }
                else
                {
                    command.Items.Add( child );
                }
            }
        }

        private void ShowFileCommandSpecializedPanel()
        {
            splitContainerSubClassSpecialized.Panel1Collapsed = false;
            splitContainerSubClassSpecialized.Panel2Collapsed = true;
        }

        private void ShowGroupCommandSpecializedPanel()
        {
            splitContainerSubClassSpecialized.Panel1Collapsed = true;
            splitContainerSubClassSpecialized.Panel2Collapsed = false;
        }

        private void okCancelPanel1_OKButton_Click( object sender, EventArgs e )
        {
            Normalize();

            if ( IsValid() )
            {
                ReturnOK();
            }
        }

        private void ReturnOK()
        {
            SetResult();

            DialogResult = DialogResult.OK;
        }

        private bool IsValid()
        {
            if ( labeledTextBoxName.TextBox.Text ==
                 string.Empty )
            {
                MessageBox.Show( "コマンド名を入力してください。" );

                return false;
            }

            if ( IsFileCommandMode() )
            {
                string fileName = labeledTextBoxFileName.TextBox.Text;

                if ( fileName == string.Empty )
                {
                    MessageBox.Show( "ファイル名を入力してください。" );

                    return false;
                }

                if ( !File.Exists( fileName ) )
                {
                    MessageBox.Show( fileName + " が見つかりません。" );

                    return false;
                }
            }

            return true;
        }

        private void Normalize()
        {
            NormalizeName();

            NormalizeLabeledTextBoxText( labeledTextBoxDefaultArguments );

            NormalizeLabeledTextBoxText( labeledTextBoxFileName );
            NormalizeLabeledTextBoxText( labeledTextBoxVerb );
            NormalizeLabeledTextBoxText( labeledTextBoxWorkingDirectory );
        }

        private void NormalizeName()
        {
            NormalizeLabeledTextBoxText( labeledTextBoxName );

            labeledTextBoxName.TextBox.Text = labeledTextBoxName.TextBox.Text.Replace( StringConstants.DoubleQuotation,
                                                                                       string.Empty );
            labeledTextBoxName.TextBox.Text = labeledTextBoxName.TextBox.Text.Replace( ":", string.Empty );
        }

        private void NormalizeLabeledTextBoxText( LabeledTextBox labeledTextBox )
        {
            labeledTextBox.TextBox.Text = labeledTextBox.TextBox.Text.Trim();
        }

        private void SetResult()
        {
            if ( IsFileCommandMode() )
            {
                SetResultAsCommand( fileCommand );
                SetResultAsFileCommand( fileCommand );
            }
            else
            {
                SetResultAsCommand( groupCommand );
                SetResultAsGroupCommand( groupCommand );
            }
        }

        private bool IsFileCommandMode()
        {
            return splitContainerSubClassSpecialized.Panel2Collapsed;
        }

        private void buttonAddTag_Click( object sender, EventArgs e )
        {
            ShowAddTagForm();
        }

        private void ShowAddTagForm()
        {
            string[] contents;

            try
            {
                contents = LoadContents();
            }
            catch ( Exception e )
            {
                MessageBox.Show( "タグファイルの読み込みに失敗しました。\r\n" + e.Message );
                return;
            }

            if ( contents.Length == 0 )
            {
                MessageBox.Show( "タグファイルが空です。" );
                return;
            }

            using ( TagListForm tagListForm = new TagListForm() )
            {
                tagListForm.SetContents( contents );

                if ( tagListForm.ShowDialog() ==
                     DialogResult.OK )
                {
                    AddTag( tagListForm.Result );
                }
            }
        }

        private void AddTag( string tag )
        {
            TextBox tags = labeledTextBoxTags.TextBox;

            if ( tags.Lines.Length == 0 ||
                 tags.Lines[tags.Lines.Length - 1] == string.Empty )
            {
                tags.AppendText( tag );
            }
            else
            {
                tags.AppendText( "\r\n" + tag );
            }
        }

        private string[] LoadContents()
        {
            if ( !File.Exists( tagsFileName ) )
            {
                CreateTagsFile();
            }

            StringList file = new StringList( tagsFileName, Encoding.UTF8 );

            return file.ToArray();
        }

        private void CreateTagsFile()
        {
            if ( File.Exists( sampleTagsFileName ) )
            {
                File.Copy( sampleTagsFileName, tagsFileName );
            }
            else
            {
                StringList file = new StringList();

                file.SaveToFile( tagsFileName, Encoding.UTF8 );
            }
        }
    }
}