﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using Tomham.Xml.Serialization;
using Tomham.Collections.Generic;
using Tomham.Diagnostics;
using System.Diagnostics;
using Tomham.Windows.Forms;
using Tomham.Collections.Specialized;

namespace CommandTagLauncher
{
	public partial class FormMain : Form
	{
		public FormMain()
		{
			InitializeComponent();
		}

		private Helper helper;

		private InputArgumentsForm inputArgumentsForm;

		private bool hideWhenLoaded;

		private bool confirmWhenRemovingItems;
		private bool confirmWhenClearArgumentsHistory;

		private Keys executeKey;
		private Keys executeWithInputArgumentsKey;
		private Keys executeWithMostRecentUsedArgumentsKey;

		private void FormMain_Load( object sender, EventArgs e )
		{
			Initialize();
		}

		private void Initialize()
		{
			helper = new Helper();

			inputArgumentsForm = new InputArgumentsForm();

			LoadCommands();

			LoadOptions();

			InitializeFilterListBox1();
		}

		private void InitializeFilterListBox1()
		{
			UpdateFilterListBoxSource();

			filterListBox1.SetMatchedItemsSorter( new ItemComparer() );

			filterListBox1.IsMatchedItem = IsMatchedItem;
		}

		private void UpdateFilterListBoxSource()
		{
			filterListBox1.SetSource(commands.ToArray()  );
		}

		#region Commands
		private Commands commands;

		public Commands Commands
		{
			get { return commands; }
			set { commands = value; }
		} 
		#endregion

		#region FileNames

		private readonly string commonOptionsFileName =
			Path.Combine( Application.StartupPath, @"CommonOptions.xml" );

		private readonly string viewOptionsFileName =
			Path.Combine( Application.StartupPath, @"ViewOptions.xml" );

		private readonly string commandsFileName =
			Path.Combine( Application.StartupPath, @"Commands.xml" );

		#endregion

		#region LoadOptions
		private void LoadOptions()
		{
			LoadCommonOptions();

			LoadViewOptions();
		}

		private void LoadCommonOptions()
		{
			CommonOptions options = LoadCommonOptionsFromFile( commonOptionsFileName );

			UpplyFromCommonOptions( options );
		}

		private void UpplyFromCommonOptions( CommonOptions options )
		{
			systemHotKeyToggleMainWindowVisible.HotKey = options.ToggleMainWindowVisibleKey;
			systemHotKeyCaptureCurrentProgram.HotKey = options.CaptureCurrentProgramKey;

			executeKey = options.ExecuteKey;
			executeWithInputArgumentsKey = options.ExecuteWithInputArgumentsKey;
			executeWithMostRecentUsedArgumentsKey = options.ExecuteWithMostRecentUsedArgumentsKey;

			hideWhenLoaded = options.HideWhenLoaded;

			confirmWhenRemovingItems = options.ConfirmWhenRemovingItems;
			confirmWhenClearArgumentsHistory = options.ConfirmWhenClearArgumentsHistory;

			Commands.SetArgumentsHistoryMaxCount( options.CommandsArgumentsHistoryMaxCount );

			checkBoxInitialSearch.Checked = options.InitialSearch;
			checkBoxRomajiMatching.Checked = options.RomajiMatching;

			TopMost = options.TopMost;

			filterListBox1.MaxShowItemsCount = options.MaxShowItemsCount;
		}

		private CommonOptions LoadCommonOptionsFromFile( string fileName )
		{
			if ( File.Exists( commonOptionsFileName ) )
			{
				return SimpleXmlSerializer.Deserialize<CommonOptions>( fileName, Encoding.UTF8 );
			}

			return new CommonOptions();
		}

		private void LoadViewOptions()
		{
			ViewOptions viewOptions = LoadViewOptionsFromFile();

			UpplyFromViewOptions( viewOptions );
		}

		private void UpplyFromViewOptions( ViewOptions viewOptions )
		{
			filterListBox1.Font = viewOptions.Font;
			labelSelectedItemName.Font = viewOptions.Font;
			textBoxInformation.Font = viewOptions.Font;

			filterListBox1.TextBackColor = viewOptions.TextBackColor;
			filterListBox1.TextForeColor = viewOptions.TextForeColor;
			textBoxInformation.BackColor = viewOptions.TextBackColor;
			textBoxInformation.ForeColor = viewOptions.TextForeColor;

			inputArgumentsForm.SetView(
				viewOptions.Font,
				viewOptions.TextBackColor,
				viewOptions.TextForeColor
			);

			splitContainer1.Panel2Collapsed = viewOptions.InformationPanelCollapsed;
		}

		private ViewOptions LoadViewOptionsFromFile()
		{
			if ( File.Exists( viewOptionsFileName ) )
			{
				return DoLoadViewOptionsFromFile( viewOptionsFileName );
			}

			return new ViewOptions();
		}

		private ViewOptions DoLoadViewOptionsFromFile( string fileName )
		{
			try
			{
				Yaowi.Common.Serialization.XmlDeserializer deserializer =
						new Yaowi.Common.Serialization.XmlDeserializer();

				object data = deserializer.Deserialize( fileName );

				return ( ViewOptions )data;
			}
			catch ( Exception )
			{
				return new ViewOptions();
			}
		}

		#endregion

		private void LoadCommands()
		{
			if ( File.Exists( commandsFileName ) )
			{
				Commands = CommandsSerializer.Load( commandsFileName );
			}
			else
			{
				Commands = new Commands();
			}

			Commands.Updated += Commands_Updated;
		}

		void Commands_Updated( object sender, UpdatedEventArgs e )
		{
			UpdateFilterListBoxSource();
		}

		#region SaveFiles

		private void SaveFiles()
		{
			SaveCommands();

			SaveOptions();
		}

		private void SaveCommands()
		{
			CommandsSerializer.Save( Commands, commandsFileName );
		}

		private void SaveOptions()
		{
			SaveCommonOptions();

			SaveViewOptions();
		}

		private Yaowi.Common.Serialization.XmlSerializer deepSerializer =
			new Yaowi.Common.Serialization.XmlSerializer();

		private void SaveViewOptions()
		{
			ViewOptions viewOptions = CreateViewOptions();

			deepSerializer.Serialize( viewOptions, viewOptionsFileName );
		}

		private ViewOptions CreateViewOptions()
		{
			ViewOptions viewOptions = new ViewOptions();

			viewOptions.Font = filterListBox1.Font;

			viewOptions.TextBackColor = filterListBox1.TextBackColor;
			viewOptions.TextForeColor = filterListBox1.TextForeColor;

			viewOptions.InformationPanelCollapsed = splitContainer1.Panel2Collapsed;

			return viewOptions;
		}

		private void SaveCommonOptions()
		{
			CommonOptions options = CreateCommonOptions();

			CommonOptionsSerializer.Save( options, commonOptionsFileName );
		}

		private CommonOptions CreateCommonOptions()
		{
			CommonOptions options = new CommonOptions();

			options.ToggleMainWindowVisibleKey = systemHotKeyToggleMainWindowVisible.HotKey;
			options.CaptureCurrentProgramKey = systemHotKeyCaptureCurrentProgram.HotKey;

			options.ExecuteKey = executeKey;
			options.ExecuteWithInputArgumentsKey = executeWithInputArgumentsKey;
			options.ExecuteWithMostRecentUsedArgumentsKey = executeWithMostRecentUsedArgumentsKey;

			options.HideWhenLoaded = hideWhenLoaded;

			options.CommandsArgumentsHistoryMaxCount = Commands.GetArgumentsHistoryMaxCount();

			options.ConfirmWhenRemovingItems = confirmWhenRemovingItems;
			options.ConfirmWhenClearArgumentsHistory = confirmWhenClearArgumentsHistory;

			options.InitialSearch = checkBoxInitialSearch.Checked;
			options.RomajiMatching = checkBoxRomajiMatching.Checked;

			options.TopMost = TopMost;

			options.MaxShowItemsCount = filterListBox1.MaxShowItemsCount;

			return options;
		}
		#endregion

		private void filterListBox1_MatchedItemsUpdating( object sender, EventArgs e )
		{
			helper.PrepareIsMatchedItemArguments(
				filterListBox1.FilterText,
				checkBoxInitialSearch.Checked,
				checkBoxRomajiMatching.Checked
			);
		}

		#region IsMatchedItem

		private bool IsMatchedItem( object item )
		{
			if ( filterListBox1.FilterText == string.Empty )
			{
				return false;
			}

			Command command = ( Command )item;

			return helper.IsMatchedItem( command );
		}

		#endregion

		private Keys removeLastExpressionKey = Keys.Shift | Keys.Back;

		private void filterListBox1_TextBoxKeyDown( object sender, KeyEventArgs e )
		{
			if ( KeysCheckAndExecute( e ) )
			{
				return;
			}

			if ( e.KeyData == Keys.Down )
			{
				DoActionWithSuppresKeyPress( e, FocusToMatchedItemsListBox );
			}
			else if ( e.KeyData == Keys.Escape )
			{
				DoActionWithSuppresKeyPress( e, filterListBox1.ClearFilterText );
			}
			else if ( e.KeyData == removeLastExpressionKey )
			{
				//DoActionWithSuppresKeyPress( e, new Action( RemoveLastExpression ) );
			}
		}

		private void filterListBox1_ListBoxKeyDown( object sender, KeyEventArgs e )
		{
			if ( KeysCheckAndExecute( e ) )
			{
				return;
			}
			else if ( e.KeyData == Keys.Up )
			{
				MoveToUp();
			}
		}

		private void RemoveLastExpression()
		{
			// todo: make implementation.
		}

		private void DoActionWithSuppresKeyPress( KeyEventArgs e, Action action )
		{
			e.SuppressKeyPress = true;

			action();
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="e"></param>
		/// <returns>executed</returns>
		private bool KeysCheckAndExecute( KeyEventArgs e )
		{
			if ( e.KeyData == executeWithInputArgumentsKey )
			{
				DoActionWithSuppresKeyPress( e, ExecuteSelectedItemWithInputArguments );
			}
			else if ( e.KeyData == executeWithMostRecentUsedArgumentsKey )
			{
				DoActionWithSuppresKeyPress( e, ExecuteSelectedItemWithMostRecentUsedArguments );
			}
			else if ( e.KeyData == executeKey )
			{
				DoActionWithSuppresKeyPress( e, ExecuteSelectedItemWithDefaultArguments );
			}
			else
			{
				return false;
			}

			return true;
		}

		private void Execute( Command command, string arguments )
		{
			command.Execute( arguments );

			Hide();
		}

		private void ExecuteSelectedItemWithMostRecentUsedArguments()
		{
			Command command = GetOnlyOneSelectedItem();

			if ( command != null )
			{
				string arguments = command.DefaultArguments;

				if ( 0 < command.ArgumentsHistory.Count )
				{
					arguments = command.ArgumentsHistory[0];
				}

				Execute( command, arguments );
			}
		}

		private void ExecuteSelectedItemWithDefaultArguments()
		{
			Command command = GetOnlyOneSelectedItem();

			if ( command != null )
			{
				Execute( command, command.DefaultArguments );
			}
		}

		private void ExecuteSelectedItemWithInputArguments()
		{
			DoActionWithFirmedWindowVisible( DoExecuteSelectedItemWithInputArguments );
		}

		private void DoExecuteSelectedItemWithInputArguments()
		{
			Command command = GetOnlyOneSelectedItem();

			if ( command != null )
			{
				inputArgumentsForm.SetCommand( command );

				if ( inputArgumentsForm.ShowDialog() == DialogResult.OK )
				{
					Execute( command, inputArgumentsForm.Arguments );
				}
			}
		}

		private void MoveToUp()
		{
			if ( IsOnTheFirstItemOfMatchedItemsListBox() )
			{
				filterListBox1.FocusToTextBox();
			}
		}

		private bool IsOnTheFirstItemOfMatchedItemsListBox()
		{
			return filterListBox1.GetSelectedItem<object>() ==
				filterListBox1.GetFirstItem<object>();
		}

		private void FocusToMatchedItemsListBox()
		{
			filterListBox1.ClearSelected();

			if ( 2 <= filterListBox1.GetMatchedItemsCount() )
			{
				filterListBox1.SetSelected( 1, true );
				filterListBox1.FocusToListBox();
			}
			else if ( filterListBox1.HasMatchedItems() )
			{
				filterListBox1.SelectFirstItem();
				filterListBox1.FocusToListBox();
			}
		}

		private bool enabledChangeMainWindowVisible = true;

		private void systemHotKeyToggleMainWindowVisible_Pressed( object sender, EventArgs e )
		{
			if ( enabledChangeMainWindowVisible )
			{
				ToggleMainWindowVisible();
			}
		}

		private void ToggleMainWindowVisible()
		{
			if ( Visible )
			{
				Hide();
			}
			else
			{
				ShowAndActivate();
			}
		}

		private bool exitFromUser;

		private void FormMain_FormClosing( object sender, FormClosingEventArgs e )
		{
			if ( systemShutDown || exitFromUser )
			{
				TearDown();
			}
			else
			{
				e.Cancel = true;

				Hide();
			}
		}

		private void toolStripMenuItemExit_Click( object sender, EventArgs e )
		{
			Exit();
		}

		private void Exit()
		{
			exitFromUser = true;

			Close();
		}

		#region TearDown

		private void TearDown()
		{
			HideNotifyIcon();

			SaveFiles();

			inputArgumentsForm.Dispose();
		}

		private void HideNotifyIcon()
		{
			notifyIcon1.Visible = false;
		}

		#endregion

		private void ShowAndActivate()
		{
			if ( Visible == false )
			{
				Show();

				filterListBox1.ClearFilterText();

				FocusToFilterListBox1();
			}

			Utility.ActivateWindow( Process.GetCurrentProcess() );
		}

		private void notifyIcon1_MouseClick( object sender, MouseEventArgs e )
		{
			if ( enabledChangeMainWindowVisible &&
				e.Button == MouseButtons.Left )
			{
				ShowAndActivate();
			}
		}

		private void FocusToFilterListBox1()
		{
			filterListBox1.Focus();

			filterListBox1.FocusToTextBox();
		}

		private void FocusToFilterListBox1AndUpdateMatchedItems()
		{
			FocusToFilterListBox1();

			filterListBox1.UpdateMatchedItems();
		}

		private void checkBoxInitialSearch_CheckedChanged( object sender, EventArgs e )
		{
			FocusToFilterListBox1AndUpdateMatchedItems();
		}

		private void checkBoxRomajiMatching_CheckedChanged( object sender, EventArgs e )
		{
			FocusToFilterListBox1AndUpdateMatchedItems();
		}

		bool systemShutDown;

		protected override void WndProc( ref Message m )
		{
			const int WM_SYSCOMMAND = 0x0112;
			const int SC_MINIMIZE = 0xf020;
			const int WM_QUERYENDSESSION = 0x11;

			if ( ( m.Msg == WM_SYSCOMMAND ) &&
				( ( ( int )m.WParam & 0xfff0 ) == SC_MINIMIZE ) )
			{
				Hide();

				return;
			}

			if ( m.Msg == WM_QUERYENDSESSION )
			{
				systemShutDown = true;
			}

			base.WndProc( ref m );
		}

		private void FormMain_Shown( object sender, EventArgs e )
		{
			if ( hideWhenLoaded )
			{
				Hide();
			}
			else
			{
				FocusToFilterListBox1();
			}
		}

		private void toolStripMenuItemColse_Click( object sender, EventArgs e )
		{
			Exit();
		}

		private void toolStripMenuItemDisplayAllTags_Click( object sender, EventArgs e )
		{
			DisplayAllTags();
		}

		private void filterListBox1_MatchedItemsUpdated( object sender, EventArgs e )
		{
			filterListBox1.SelectFirstItem();

			UpdateSelectedItemInformations();
		}

		private void filterListBox1_SelectedIndexChanged( object sender, EventArgs e )
		{
			UpdateSelectedItemInformations();
		}

		private void UpdateSelectedItemInformations()
		{
			UpdateLabelSelectedItemName();

			UpdateInformation();
		}

		private void UpdateInformation()
		{
			Command selectedItem = GetOnlyOneSelectedItem();

			if ( selectedItem == null )
			{
				textBoxInformation.Text = string.Empty;
			}
			else
			{
				string information = GetInformation( selectedItem );

				textBoxInformation.Text = information;
			}
		}

		private string newLine = "\r\n";

		private string GetInformation( Command selectedItem )
		{
			StringBuilder result = new StringBuilder();

			result.Append( "説明:" );
			result.Append( newLine );
			result.Append( selectedItem.Description );
			result.Append( newLine + newLine );

			result.Append( "タグ:" );
			result.Append( newLine );

			foreach ( string tag in selectedItem.Tags )
			{
				result.Append( tag + newLine );
			}

			result.Append( newLine );

			if ( 1 <= selectedItem.ArgumentsHistory.Count )
			{
				result.Append( "最新の引数履歴:" );
				result.Append( newLine );
				result.Append( selectedItem.ArgumentsHistory[0] );
			}

			return result.ToString();
		}

		private void UpdateLabelSelectedItemName()
		{
			Command selectedItem = GetOnlyOneSelectedItem();

			if ( selectedItem == null )
			{
				ClearLabelSelectedItemName();
			}
			else
			{
				labelSelectedItemName.Text = selectedItem.Name;
			}
		}

		private Command GetOnlyOneSelectedItem()
		{
			if ( filterListBox1.GetSelectedItemsCount() == 1 )
			{
				return filterListBox1.GetSelectedItem<Command>();
			}

			return null;
		}

		private void ClearLabelSelectedItemName()
		{
			labelSelectedItemName.Text = "No Match";
		}

		private void toolStripMenuItemInformation_Click( object sender, EventArgs e )
		{
			ToggleInformationPanelCollapsed();
		}

		private void ToggleInformationPanelCollapsed()
		{
			splitContainer1.Panel2Collapsed = !splitContainer1.Panel2Collapsed;
		}

		private void toolStripMenuItemView_Paint( object sender, PaintEventArgs e )
		{
			UpdateViewSubMenuItemsChecked();
		}

		private void UpdateViewSubMenuItemsChecked()
		{
			toolStripMenuItemInformation.Checked = !splitContainer1.Panel2Collapsed;
			toolStripMenuItemTopMost.Checked = TopMost;
		}

		private void toolStripMenuItemTopMost_Click( object sender, EventArgs e )
		{
			TopMost = !TopMost;
		}

		private void toolStripMenuItemVersion_Click( object sender, EventArgs e )
		{
			DoActionWithFirmedWindowVisible( ShowVersion );
		}

		private void ShowVersion()
		{
			MessageBox.Show( "1.05" );
		}

		private void toolStripMenuRemoveItems_Click( object sender, EventArgs e )
		{
			RemoveItems();
		}

		private void RemoveItems()
		{
			DoActionWithFirmedWindowVisible( DoRemoveItems );
		}

		private void DoRemoveItems()
		{
			List<Command> removeTargets = filterListBox1.GetSelectedItems<Command>();

			if ( 0 < removeTargets.Count )
			{
				if ( confirmWhenRemovingItems )
				{
					if ( ShowConfirmBox( "選択したアイテムを削除してよろしいですか？" ) )
					{
						DoRemoveItems( removeTargets );
					}
				}
				else
				{
					DoRemoveItems( removeTargets );
				}
			}
		}

		private bool ShowConfirmBox( string message )
		{
			DialogResult dialogResult = MessageBox.Show(
				this,
				message,
				"確認",
				MessageBoxButtons.OKCancel,
				MessageBoxIcon.Question,
				MessageBoxDefaultButton.Button1
			);

			return dialogResult == DialogResult.OK;
		}

		private void DoRemoveItems( List<Command> removeTargets )
		{
			foreach ( Command command in removeTargets )
			{
				Commands.Remove( command );
			}
		}

		private void toolStripMenuItemOptions_Click( object sender, EventArgs e )
		{
			EditOptions();
		}

		private void EditOptions()
		{
			DoActionWithFirmedWindowVisible( DoEditOptions );
		}

		private void DoEditOptions()
		{
			CommonOptions commonOptions = CreateCommonOptions();

			ViewOptions viewOptions = CreateViewOptions();

			OptionsForm form = new OptionsForm( commonOptions, viewOptions );

			try
			{
				if ( form.ShowDialog() == DialogResult.OK )
				{
					UpplyFromCommonOptions( form.CommonOptions );
					UpplyFromViewOptions( form.ViewOptions );
				}
			}
			finally
			{
				form.Dispose();
			}
		}

		private void DoActionWithFirmedWindowVisible( Action action )
		{
			enabledChangeMainWindowVisible = false;

			try
			{
				action();
			}
			finally
			{
				enabledChangeMainWindowVisible = true;
			}
		}

		#region DisplayAllTags

		private void DisplayAllTags()
		{
			DoActionWithFirmedWindowVisible( DoDisplayAllTags );
		}

		private void DoDisplayAllTags()
		{
			List<string> tags = GetAllTags();

			StringBuilder builder = new StringBuilder();

			foreach ( string tag in tags )
			{
				builder.Append( tag + "\n" );
			}

			MessageBox.Show( builder.ToString() );
		}

		private List<string> GetAllTags()
		{
			List<string> result = new List<string>();

			foreach ( Command command in commands )
			{
				foreach ( string tag in command.Tags )
				{
					if ( !result.Contains( tag ) )
					{
						result.Add( tag );
					}
				}
			}

			result.Sort();

			return result;
		}
		#endregion

		private void NewFileCommand()
		{
			EditAndAddCommand( new FileCommand() );
		}

		private void NewGroupCommand()
		{
			EditAndAddCommand( new GroupCommand() );
		}

		private void EditAndAddCommand( Command command )
		{
			editTarget = command;

			DoActionWithFirmedWindowVisible( DoEditAndAddCommand );
		}

		private Command editTarget;

		private void DoEditAndAddCommand()
		{
			List<Command> childCommandsReferencesResolveSource =
				IListUtility.ExtractToList<Command>( Commands, IsNotGroupCommand );

			CommandEditForm editForm = new CommandEditForm();

			editForm.SetChildCommandsReferencesResolveSource( childCommandsReferencesResolveSource );

			editForm.SetCommand( editTarget );

			try
			{
				if ( editForm.ShowDialog() == DialogResult.OK )
				{
					Commands.Add( editTarget );
				}
			}
			finally
			{
				editForm.Dispose();
			}
		}

		private bool IsNotGroupCommand( Command command )
		{
			return !( command is GroupCommand );
		}

		#region toolStripMenuItemClick Events
		private void toolStripMenuItemNewFileCommand_Click( object sender, EventArgs e )
		{
			NewFileCommand();
		}

		private void toolStripMenuItemNewGroupCommand_Click( object sender, EventArgs e )
		{
			NewGroupCommand();
		}

		private void toolStripMenuItemEditItem_Click( object sender, EventArgs e )
		{
			EditItem();
		}
		#endregion

		private void EditItem()
		{
			DoActionWithFirmedWindowVisible( DoEditItem );
		}

		private void DoEditItem()
		{
			if ( filterListBox1.GetSelectedItemsCount() == 1 )
			{
				Command command = filterListBox1.GetSelectedItem<Command>();

				CommandEditForm editForm = new CommandEditForm();

				editForm.SetCommand( command );

				List<Command> childCommandsReferencesResolveSource =
					IListUtility.ExtractToList<Command>( Commands, IsNotGroupCommand );

				editForm.SetChildCommandsReferencesResolveSource( childCommandsReferencesResolveSource );

				try
				{
					editForm.ShowDialog();
				}
				finally
				{
					editForm.Dispose();
				}
			}

			UpdateInformation();
		}

		private void systemHotKeyToggleMainWindowVisible_HotKeyChanged( object sender, EventArgs e )
		{
			string text = "CommandTagLauncher\r\n" + "表示・非表示キー: " + systemHotKeyToggleMainWindowVisible.HotKey.ToString();

			notifyIcon1.Text = text;
		}

		private Keys focusToFilterListBox1Key = Keys.Control | Keys.F;

		private void FormMain_KeyDown( object sender, KeyEventArgs e )
		{
			if ( e.KeyData == focusToFilterListBox1Key )
			{
				FocusToFilterListBox1();

				e.SuppressKeyPress = true;
			}
		}

		private void FormMain_Deactivate( object sender, EventArgs e )
		{
			if ( enabledChangeMainWindowVisible )
			{
				Hide();
			}
		}

		private void systemHotKeyCaptureCurrentProgram_HotKeyPressed( object sender, EventArgs e )
		{
			CaptureCurrentProgram();
		}

		private void CaptureCurrentProgram()
		{
			if ( Visible )
			{
				return;
			}

			try
			{
				DoCaptureCurrentProgram();
			}
			catch ( Exception e )
			{
				MessageBox.Show( e.Message );
			}
		}

		private void DoCaptureCurrentProgram()
		{
			FileCommand command = CreateForegroundWindowFileCommand();

			if ( command != null )
			{
				ShowAndActivate();

				EditAndAddCommand( command );

				Hide();
			}
		}

		private FileCommand CreateForegroundWindowFileCommand()
		{
			FileCommand command = new FileCommand();

			try
			{
				Process foreground = Utility.GetForegroundWindowProcess();

				command.Name = CreateName( foreground );
				command.FileName = foreground.MainModule.FileName;

				return command;
			}
			catch ( Exception e )
			{
				MessageBox.Show( e.Message );

				return null;
			}
		}

		private string CreateName( Process process )
		{
			if ( process.MainModule.FileVersionInfo.ProductName != string.Empty )
			{
				return process.MainModule.FileVersionInfo.ProductName;
			}

			return Path.GetFileName( process.MainModule.FileName );
		}

		private void RegisterHotKeyFailed( object sender, RegisterHotKeyFailedEventArgs e )
		{
			StringBuilder message = new StringBuilder();

			message.Append( "ホットキー [ " + e.FailedKey.ToString() + " ] の登録中にエラーが発生しました。\n" );
			message.Append( "システムの他のホットキーと競合しているのかもしれません。\n" );
			message.Append( "他のキーに変更してください。" );

			MessageBox.Show( message.ToString() );
		}

		private void toolStripMenuItemCreateTagsFileFromAllTags_Click( object sender, EventArgs e )
		{
			CreateTagsFileFromAllTags();
		}

		private void CreateTagsFileFromAllTags()
		{
			using ( SaveFileDialog dialog = new SaveFileDialog() )
			{
				dialog.DefaultExt = ".txt";

				dialog.Filter = "Text files (*.txt)|*.txt";

				if ( dialog.ShowDialog( this ) == DialogResult.OK )
				{
					List<string> tags = GetAllTags();

					StringList file = new StringList();

					file.AddRange( tags );

					file.SaveToFile( dialog.FileName, Encoding.UTF8 );
				}
			}
		}

		private void toolStripMenuItemClearArgumentsHistory_Click( object sender, EventArgs e )
		{
			ClearArgumentsHistory();
		}

		private void ClearArgumentsHistory()
		{
			DoActionWithFirmedWindowVisible( DoClearArgumentsHistory );
		}

		private void DoClearArgumentsHistory()
		{
			List<Command> targets = filterListBox1.GetSelectedItems<Command>();

			if ( 0 < targets.Count )
			{
				if ( confirmWhenClearArgumentsHistory )
				{
					if ( ShowConfirmBox( "選択したアイテムの引数履歴をクリアしてよろしいですか？" ) )
					{
						DoClearArgumentsHistory( targets );
					}
				}
				else
				{
					DoClearArgumentsHistory( targets );
				}
			}
		}

		private void DoClearArgumentsHistory( List<Command> targets )
		{
			foreach ( Command command in targets )
			{
				command.ClearArgumentsHistory();
			}

			UpdateInformation();
		}
	}
}
