﻿using System;
using System.Collections.Generic;
using System.ComponentModel;

namespace CommandTagLauncher
{
	public abstract class Command 
	{
		private string name = string.Empty;

		public string Name
		{
			get { return name; }
			set
			{
				name = value;
				NotifyObservers();
			}
		}

		private string description = string.Empty;

		[DefaultValue( "" )]
		public string Description
		{
			get { return description; }
			set { description = value; }
		}

		private DateTime lastExecutedTime = DateTime.Now;

		public DateTime LastExecutedTime
		{
			get { return lastExecutedTime; }
			set { lastExecutedTime = value; }
		}

		private List<string> tags = new List<string>();

		public List<string> Tags
		{
			get { return tags; }
			set { tags = value; }
		}

		private string defaultArguments;

		[DefaultValue( "" )]
		public string DefaultArguments
		{
			get
			{
				if ( defaultArguments == null )
				{
					return string.Empty;
				}

				return defaultArguments;
			}

			set { defaultArguments = value; }
		}

		private List<string> argumentsHistory = new List<string>();

		public List<string> ArgumentsHistory
		{
			get { return argumentsHistory; }
			set { argumentsHistory = value; }
		}

		public void Execute( string arguments )
		{
			string trimmedArguments = arguments.Trim();

			DoExecute( trimmedArguments );

			LastExecutedTime = DateTime.Now;

			UpdateHistory( trimmedArguments );
		}

		private void UpdateHistory( string arguments )
		{
			if ( arguments == string.Empty )
			{
				return;
			}

			if ( ArgumentsHistory.Contains( arguments ) )
			{
				MoveToFirst( arguments );
			}
			else
			{
				ArgumentsHistory.Insert( 0, arguments );
			}

			AdjustArgumentsHistoryCount();
		}

		private void AdjustArgumentsHistoryCount()
		{
			if ( argumentsHistoryMaxCount < ArgumentsHistory.Count )
			{
				ArgumentsHistory.RemoveRange(
					argumentsHistoryMaxCount,
					ArgumentsHistory.Count - argumentsHistoryMaxCount
				);
			}
		}

		private void MoveToFirst( string arguments )
		{
			ArgumentsHistory.Remove( arguments );

			ArgumentsHistory.Insert( 0, arguments );
		}

		protected abstract void DoExecute( string arguments );

		private readonly List<ICommandObserver> observers = new List<ICommandObserver>();

		public void AddObserver( ICommandObserver observer )
		{
			observers.Add( observer );
		}

		public void DeleteObserver( ICommandObserver observer )
		{
			observers.Remove( observer );
		}

		protected void NotifyObservers()
		{
			foreach ( ICommandObserver observer in observers )
			{
				observer.Update( this );
			}
		}

		public const int DefaultArgumentsHistoryMaxCount = 32;

		private int argumentsHistoryMaxCount = DefaultArgumentsHistoryMaxCount;

		public int GetArgumentsHistoryMaxCount()
		{
			return argumentsHistoryMaxCount;
		}

		public void SetArgumentsHistoryMaxCount( int value )
		{
			if ( value < 0 )
			{
				argumentsHistoryMaxCount = DefaultArgumentsHistoryMaxCount;
			}
			else
			{
				argumentsHistoryMaxCount = value;
			}

			AdjustArgumentsHistoryCount();
		}

		public void ClearArgumentsHistory()
		{
			ArgumentsHistory.Clear();
		}

		public override string ToString()
		{
			return Name;
		}
	}
}
