﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Drawing;
using System.IO;
using System.Xml.Serialization;

namespace Tomham.Windows.Forms
{
	public partial class FormStateFixer : Component
	{
		public FormStateFixer()
		{
			InitializeComponent();
		}

		public FormStateFixer( IContainer container )
		{
			container.Add( this );

			InitializeComponent();
		}

		private Form target;

		[Browsable( true )]
		public Form Target
		{
			get
			{
				return target;
			}
			set
			{
				SetTarget( value );
			}
		}

		private void SetTarget( Form value )
		{
			if ( DesignMode )
			{
				target = value;
				return;
			}

			if ( target == value )
			{
				return;
			}

			if ( target != null )
			{
				RemoveEventsFromTarget();
			}

			target = value;

			if ( target != null )
			{
				AddEventsToTarget();
			}
		}

		private IFixer fixer = new XMLFixer();

		/// <summary>
		/// if you want to save to Registry or IniFile or others, Set your customized class.
		/// </summary>
		/// <remarks>
		/// if you want to set this property, Do it before Target's Load event raised.
		/// </remarks>
		[Browsable(false)]
		public IFixer Fixer
		{
			get { return fixer; }
			set { fixer = value; }
		}

		private void RemoveEventsFromTarget()
		{
			target.Load -= new EventHandler( FormLoad );
			target.FormClosed -= new FormClosedEventHandler( FormClosed );
		}

		private void AddEventsToTarget()
		{
			target.Load += new EventHandler( FormLoad );
			target.FormClosed += new FormClosedEventHandler( FormClosed );
		}

		private void FormLoad( object sender, EventArgs e )
		{
			Restore();
		}

		private void Restore()
		{
			fixer.Restore( target );
		}

		private void FormClosed( object sender, EventArgs e )
		{
			Save();
		}

		private void Save()
		{
			FormState state = GetFormstate();

			fixer.Save( state );
		}

		private FormState GetFormstate()
		{
			FormState state = new FormState();

			state.WindowState = target.WindowState;

			if ( target.WindowState != FormWindowState.Normal )
			{
				target.WindowState = FormWindowState.Normal;
			}

			state.Size = target.Size;
			state.Location = target.Location;

			return state;
		}

		public class FormState
		{
			private Size size;

			public Size Size
			{
				get { return size; }
				set { size = value; }
			}

			private Point location;

			public Point Location
			{
				get { return location; }
				set { location = value; }
			}

			private FormWindowState windowState;

			public FormWindowState WindowState
			{
				get { return windowState; }
				set { windowState = value; }
			}
		}

		public interface IFixer
		{
			void Save( FormState state );
			void Restore( Form target );
		}

		public class XMLFixer : IFixer
		{
			private string fileName = @"Windowstate.xml";

			public string FileName
			{
				get { return fileName; }
				set { fileName = value; }
			}

			private Encoding encoding = Encoding.UTF8;

			public Encoding Encoding
			{
				get { return encoding; }
				set { encoding = value; }
			}

			private XmlSerializer serializer = new XmlSerializer( typeof( FormState ) );

			#region IFixer メンバ

			public void Save( FormState state )
			{
				using ( TextWriter writer = new StreamWriter( fileName, false, encoding ) )
				{
					serializer.Serialize( writer, state );
				}
			}

			public void Restore( Form target )
			{
				if ( !File.Exists( fileName ) )
				{
					return;
				}

				FormState state = LoadState();

				SetState( target, state );
			}

			private static void SetState( Form target, FormState state )
			{
				target.Size = state.Size;
				target.Location = state.Location;
				target.WindowState = state.WindowState;
			}

			private FormState LoadState()
			{
				FormState state;

				using ( TextReader reader = new StreamReader( fileName, encoding ) )
				{
					state = ( FormState )serializer.Deserialize( reader );
				}

				return state;
			}

			#endregion
		}

	}
}
