using System;

namespace Tomham
{
	public class Counter
	{
		#region Properties

		private readonly int maxCount;
		private int count;
		private bool enabled = true;
		private bool loop = true;
		private int interval;

		public int Count
		{
			get { return count; }
			set
			{
				if ( MaxCount < value )
				{
					throw new ArgumentOutOfRangeException( "value greater than MaxCount." );
				}
				else
				{
					count = value;

					TempClear();
				}
			}
		}

		public int MaxCount
		{
			get { return maxCount; }
		}

		public int Interval
		{
			get { return interval; }
			set
			{
				if ( value < 1 )
				{
					throw new ArgumentOutOfRangeException( "value less than 1." );
				}
				else
				{
					interval = value;
				}
			}
		}

		public bool Loop
		{
			get { return loop; }
			set { loop = value; }
		}

		public bool Enabled
		{
			get { return enabled; }
			set
			{
				enabled = value;

				if ( !enabled )
				{
					OnStopped( new CounterStopEventArgs() );
				}
			}
		}

		public event EventHandler<CounterStopEventArgs> Stopped;

		protected virtual void OnStopped( CounterStopEventArgs e )
		{
			if ( Stopped != null )
			{
				Stopped( this, e );
			}
		}

		#endregion

		#region Constructores

		public Counter()
			: this( int.MaxValue, 1, true )
		{
		}

		public Counter( int maxCount )
			: this( maxCount, 1, true )
		{
		}

		public Counter( int maxCount, int interval )
			: this( maxCount, interval, true )
		{
		}

		public Counter( int maxCount, int interval, bool loop )
		{
			if ( maxCount < 1 )
			{
				throw new ArgumentOutOfRangeException( "maxCount less than 1." );
			}

			if ( interval < 1 )
			{
				throw new ArgumentOutOfRangeException( "interval less than 1." );
			}

			this.maxCount = maxCount;
			this.interval = interval;
			this.loop = loop;
		}

		#endregion

		private int temp;

		public void Update()
		{
			if ( enabled )
			{
				DoUpdate();
			}
		}

		protected virtual void DoUpdate()
		{
			temp++;

			if ( interval <= temp )
			{
				Increment();

				TempClear();
			}
		}

		private void Increment()
		{
			if ( maxCount <= count )
			{
				if ( loop )
				{
					ReStart();
				}
				else
				{
					Stop();
				}
			}
			else
			{
				count++;
			}
		}

		private void TempClear()
		{
			temp = 0;
		}

		public void ReStart()
		{
			ReSet();

			Resume();
		}

		public void ReSet()
		{
			Count = 0;
		}

		public bool IsMaxCount()
		{
			return count == maxCount;
		}

		public void Resume()
		{
			Enabled = true;
		}

		public void Stop()
		{
			Enabled = false;
		}
	}
}